/*
 ATtiny13 LiFePO4 Charger
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/02/2018
 Last Modified: 19/03/2018
 
 Recharge a single LiFePO4 battery, controlled by an ATtiny13.
 
 */

#define F_CPU 1000000 // 1 MHz

#define lifepo4ADC 3
#define chargingMosfet PB2
#define sensingVoltageMosfet PB1
#define inputSource PB4

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"

// ATtiny13A Pin map
//
//                                         +-\/-+
//                             Reset PB5  1|o   |8  VCC
//      39K / 10K for battery (ADC3) PB3  2|    |7  PB2 N mosfet to switch on battery charging
//   8.2K / 10K divider for 5V input PB4  3|    |6  PB1 N mosfet for resistor divider on battery for measurement
//                                   GND  4|    |5  PB0 
//                                         +----+

uint8_t batteryCharged = false;

int main(void) {
	setup();
	
	while(1) {
		// If 5V source is connected
		if (PINB & (1<<inputSource)) {
			
			// Check LiFePo4 battery voltage
			PORTB |= (1<<sensingVoltageMosfet);
			sbi(ADCSRA, ADEN); // Enable ADC
			adc_read(ADC_REF_1_1V, lifepo4ADC); // Ignore first reading
			int batteryReading = adc_read(ADC_REF_1_1V, lifepo4ADC);
			cbi(ADCSRA, ADEN); // Disable ADC
			PORTB &= ~(1<<sensingVoltageMosfet);
			
			// If battery hasn't been charged yet
			if (batteryCharged == false) {
				if (batteryReading < 670) { // Less than 3.5V on LiFePo4, keep charging
					PORTB |= (1<<chargingMosfet);
				}
			}
			else { // Battery was charged, only start recharging again once battery drops below 3.2V
				if (batteryReading < 610) { // Less than 3.2V on LiFePo4, start charging
					PORTB |= (1<<chargingMosfet);
				}
			}
			
			// Battery is charged if reads more than 3.5V
			if (batteryReading >= 670) {
				PORTB &= ~(1<<chargingMosfet);
				batteryCharged = true;
			}
		}
		else {
			PORTB &= ~(1<<chargingMosfet);
		}
		
		sbi(PCMSK, PCINT4); // Enable pin change interrupt
		watchdog_sleep(T4S);
		cbi(PCMSK, PCINT4); // Disable pin change interrupt
	}
}

EMPTY_INTERRUPT(ADC_vect);
EMPTY_INTERRUPT(WDT_vect);
EMPTY_INTERRUPT(PCINT0_vect);
